"""Scenario."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Optional

from .request import Request
from .response_description import (
    ResponseDescription,
    ResponseVerification,
)
from .verification import (
    Status,
    Verification,
    merge_statuses,
)


@dataclass
class ScenarioVerification:
    status: Status
    request: Verification
    response: Optional[ResponseVerification] = None
    label: Optional[str] = None


class Scenario:
    """
    >>> from unittest.mock import MagicMock
    >>> scenario = Scenario(
    ...     request=MagicMock(Request, side_effect=RuntimeError('message')),
    ...     response_description=MagicMock(ResponseDescription),
    ... )
    >>> scenario.label
    >>> verification = scenario('base-url')
    >>> scenario.request.call_args
    call('base-url')
    >>> scenario.response_description.call_count
    0
    >>> verification.label
    >>> verification.status.name
    'FAILURE'
    >>> verification.request.status.name
    'FAILURE'
    >>> verification.request.message
    'RuntimeError: message'

    >>> from .request import Response
    >>> inner_response = MagicMock(Response, status_code=402, body='body')
    >>> scenario = Scenario(
    ...     label='Response should be unstable',
    ...     request=MagicMock(Request, return_value=inner_response),
    ...     response_description=MagicMock(
    ...         ResponseDescription,
    ...         return_value=ResponseVerification(
    ...             status=Status.UNSTABLE,
    ...             status_code=Verification.succeed(),
    ...             body=Verification(status=Status.UNSTABLE)
    ...         ),
    ...     ),
    ... )
    >>> verification = scenario('base-url')
    >>> scenario.response_description.call_args
    call(body='body', status_code=402)
    >>> verification.label
    'Response should be unstable'
    >>> verification.status.name
    'UNSTABLE'
    >>> verification.request.status.name
    'SUCCESS'
    >>> verification.response.status.name
    'UNSTABLE'
    >>> verification.response.body.status.name
    'UNSTABLE'
    """
    def __init__(
        self: Scenario,
        request: Request,
        response_description: ResponseDescription,
        label: Optional[str] = None,
    ) -> None:
        self._label = label
        self._request = request
        self._response_description = response_description

    def __call__(self: Scenario, base_url: str) -> ScenarioVerification:
        try:
            response = self._request(base_url)
        except Exception as error:
            return ScenarioVerification(
                status=Status.FAILURE,
                request=Verification.of_error(error),
            )
        request_verification = Verification.succeed()

        response_verification = self._response_description(
            status_code=response.status_code,
            body=response.body,
        )

        status = merge_statuses([
            request_verification.status,
            response_verification.status,
        ])
        return ScenarioVerification(
            status=status,
            request=request_verification,
            response=response_verification,
            label=self._label,
        )

    @property
    def label(self: Scenario) -> Optional[str]:
        return self._label

    @property
    def request(self: Scenario) -> Request:
        return self._request

    @property
    def response_description(self: Scenario) -> ResponseDescription:
        return self._response_description



"""Verification."""

from __future__ import annotations
from collections.abc import Iterable
from dataclasses import dataclass
from enum import Enum
from functools import reduce, singledispatch
from typing import Collection, Optional


class Status(Enum):
    """
    >>> Status.SUCCESS.is_succeeded
    True
    >>> Status.SUCCESS.merge(Status.SUCCESS).name
    'SUCCESS'
    >>> Status.SUCCESS.merge(Status.UNSTABLE).name
    'UNSTABLE'
    >>> Status.SUCCESS.merge(Status.FAILURE).name
    'FAILURE'

    >>> Status.UNSTABLE.is_succeeded
    False
    >>> Status.UNSTABLE.merge(Status.SUCCESS).name
    'UNSTABLE'
    >>> Status.UNSTABLE.merge(Status.UNSTABLE).name
    'UNSTABLE'
    >>> Status.UNSTABLE.merge(Status.FAILURE).name
    'FAILURE'

    >>> Status.FAILURE.is_succeeded
    False
    >>> Status.FAILURE.merge(Status.SUCCESS).name
    'FAILURE'
    >>> Status.FAILURE.merge(Status.UNSTABLE).name
    'FAILURE'
    >>> Status.FAILURE.merge(Status.FAILURE).name
    'FAILURE'
    """
    # Numbers stand for the priorities for merging.
    SUCCESS = 0
    UNSTABLE = 1
    FAILURE = 2

    @property
    def is_succeeded(self):
        return self is Status.SUCCESS

    def merge(self, other: Status):
        return max(self, other, key=lambda status: status.value)


@singledispatch
def merge_statuses(*args) -> Status:
    """
    >>> merge_statuses(1)
    Traceback (most recent call last):
        ...
    ValueError: (1,)

    For varargs.
    >>> merge_statuses(Status.UNSTABLE).name
    'UNSTABLE'
    >>> merge_statuses(Status.SUCCESS, Status.FAILURE, Status.UNSTABLE).name
    'FAILURE'

    For iterables.
    >>> merge_statuses([]).name
    'SUCCESS'
    >>> merge_statuses([Status.SUCCESS, Status.UNSTABLE, Status.FAILURE]).name
    'FAILURE'
    """
    raise ValueError(str(args))


@merge_statuses.register
def _merge_statuses_for_varargs(*statuses: Status):
    return merge_statuses(statuses)


@merge_statuses.register
def _merge_statuses_for_iterable(statuses: Iterable):
    return reduce(lambda lhs, rhs: lhs.merge(rhs), statuses, Status.SUCCESS)


@dataclass
class Verification:
    status: Status
    message: Optional[str] = None
    children: Collection[Verification] = tuple()

    @staticmethod
    def succeed() -> Verification:
        return Verification(status=Status.SUCCESS)

    @staticmethod
    def of_error(error: Exception) -> Verification:
        return Verification(
            status=Status.FAILURE,
            message=f'{error.__class__.__name__}: {error}',
        )



"""Description compilation."""

from collections.abc import Mapping
from typing import Any

from preacher.core.description import Description, Predicate
from .error import CompilationError
from .predicate import compile as compile_predicate
from .extraction import compile as compile_extraction


def compile(obj: Mapping) -> Description:
    """
    >>> from unittest.mock import patch, sentinel
    >>> extraction_patch = patch(
    ...     f'{__name__}.compile_extraction',
    ...     return_value=sentinel.extraction,
    ... )
    >>> predicate_patch = patch(
    ...     f'{__name__}.compile_predicate',
    ...     return_value=sentinel.predicate,
    ... )

    >>> compile({})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Description.describe ...

    >>> with extraction_patch as extraction_mock:
    ...     compile({'describe': 'foo', 'it': 'str'})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Description.it ...

    >>> with extraction_patch as extraction_mock:
    ...     compile({'describe': {}, 'it': [None]})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Predicate ...

    >>> with extraction_patch as extraction_mock, \\
    ...      predicate_patch as predicate_mock:
    ...     description = compile({
    ...         'describe': 'foo',
    ...         'it': {'key': 'value'}
    ...     })
    ...     extraction_mock.call_args
    ...     predicate_mock.call_args_list
    call({'jq': 'foo'})
    [call({'key': 'value'})]
    >>> description.extraction
    sentinel.extraction
    >>> description.predicates
    [sentinel.predicate]

    >>> with extraction_patch as extraction_mock, \\
    ...      predicate_patch as predicate_mock:
    ...     description = compile({
    ...         'describe': {'key': 'value'},
    ...         'it': [{'key1': 'value1'}, {'key2': 'value2'}]
    ...     })
    ...     extraction_mock.call_args
    ...     predicate_mock.call_args_list
    call({'key': 'value'})
    [call({'key1': 'value1'}), call({'key2': 'value2'})]
    >>> description.extraction
    sentinel.extraction
    >>> description.predicates
    [sentinel.predicate, sentinel.predicate]
    """
    extraction_obj = obj.get('describe')
    if isinstance(extraction_obj, str):
        extraction_obj = {'jq': extraction_obj}
    if not isinstance(extraction_obj, Mapping):
        raise CompilationError(
            f'Description.describe must be a mapping: {extraction_obj}'
        )
    extraction = compile_extraction(extraction_obj)

    predicate_objs = obj.get('it', [])
    if isinstance(predicate_objs, Mapping):
        predicate_objs = [predicate_objs]
    if not isinstance(predicate_objs, list):
        raise CompilationError(
            f'Description.it must be a list or a mapping: {predicate_objs}'
        )

    return Description(
        extraction=extraction,
        predicates=[_compile_predicate(obj) for obj in predicate_objs]
    )


def _compile_predicate(obj: Any) -> Predicate:
    if not isinstance(obj, str) and not isinstance(obj, Mapping):
        raise CompilationError(
            f'Predicate must be a string or a mapping: {obj}'
        )
    return compile_predicate(obj)



"""Request compilation."""

from collections.abc import Mapping
from preacher.core.request import Request
from .error import CompilationError


def compile(obj: Mapping) -> Request:
    """
    >>> compile({'path': {'key': 'value'}})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Request.path ...

    >>> compile({'params': ''})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Request.params ...

    >>> request = compile({})
    >>> request.path
    ''
    >>> request.params
    {}

    >>> request = compile({'path': '/path', 'params': {'key': 'value'}})
    >>> request.path
    '/path'
    >>> request.params
    {'key': 'value'}
    """
    path = obj.get('path', '')
    if not isinstance(path, str):
        raise CompilationError(f'Request.path must be a string: {path}')

    params = obj.get('params', {})
    if not isinstance(params, Mapping):
        raise CompilationError(f'Request.params must be a mapping: {params}')

    return Request(path=path, params=params)



"""Predicate compilation."""

from collections.abc import Mapping
from typing import Union

import hamcrest

from preacher.core.predicate import Predicate, of_hamcrest_matcher
from .error import CompilationError


_STATIC_MATCHER_MAP = {
    # For objects.
    'is_null': hamcrest.is_(hamcrest.none()),
    'is_not_null': hamcrest.is_(hamcrest.not_none()),
    # For collections.
    'is_empty': hamcrest.is_(hamcrest.empty()),
}
_VALUE_MATCHER_FUNCTION_MAP = {
    # For objects.
    'is': lambda expected: hamcrest.is_(expected),
    'equals_to': lambda expected: hamcrest.is_(hamcrest.equal_to(expected)),
    'has_length': lambda expected: hamcrest.has_length(expected),
    # For numbers.
    'is_greater_than': (
        lambda value: hamcrest.is_(hamcrest.greater_than(value))
    ),
    'is_greater_than_or_equal_to': (
        lambda value: hamcrest.is_(hamcrest.greater_than_or_equal_to(value))
    ),
    'is_less_than': (
        lambda value: hamcrest.is_(hamcrest.less_than(value))
    ),
    'is_less_than_or_equal_to': (
        lambda value: hamcrest.is_(hamcrest.less_than_or_equal_to(value))
    ),
    # For strings.
    'contains_string': lambda value: hamcrest.contains_string(value),
    'starts_with': lambda value: hamcrest.starts_with(value),
    'ends_with': lambda value: hamcrest.ends_with(value),
    'matches_regexp': lambda value: hamcrest.matches_regexp(value),
}
_PREDICATE_KEYS = frozenset(_VALUE_MATCHER_FUNCTION_MAP.keys())


def compile(obj: Union[str, Mapping]) -> Predicate:
    """
    >>> compile('invalid_key')
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: ... 'invalid_key'

    >>> predicate = compile('is_null')
    >>> predicate(None).status.name
    'SUCCESS'
    >>> predicate('SUCCESS').status.name
    'UNSTABLE'

    >>> predicate = compile('is_not_null')
    >>> predicate(None).status.name
    'UNSTABLE'
    >>> predicate('UNSTABLE').status.name
    'SUCCESS'

    >>> predicate = compile('is_empty')
    >>> predicate(None).status.name
    'UNSTABLE'
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate('').status.name
    'SUCCESS'
    >>> predicate([]).status.name
    'SUCCESS'
    >>> predicate('A').status.name
    'UNSTABLE'
    >>> predicate([1]).status.name
    'UNSTABLE'

    >>> compile({})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: ... has 0

    >>> compile({'equals_to': 0, 'not': {'equal_to': 1}})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: ... has 2

    >>> predicate = compile({'has_length': 1})
    >>> predicate(None).status.name
    'UNSTABLE'
    >>> predicate('').status.name
    'UNSTABLE'
    >>> predicate([]).status.name
    'UNSTABLE'
    >>> predicate('A').status.name
    'SUCCESS'
    >>> predicate([1]).status.name
    'SUCCESS'

    >>> compile({'invalid_key': 0})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: ... 'invalid_key'

    >>> predicate = compile({'is': 1})
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate('1').status.name
    'UNSTABLE'
    >>> predicate(1).status.name
    'SUCCESS'

    >>> predicate = compile({'equals_to': 1})
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate('1').status.name
    'UNSTABLE'
    >>> predicate(1).status.name
    'SUCCESS'

    >>> predicate = compile({'is_greater_than': 0})
    >>> predicate(-1).status.name
    'UNSTABLE'
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate(1).status.name
    'SUCCESS'

    >>> predicate = compile({'is_greater_than_or_equal_to': 0})
    >>> predicate(-1).status.name
    'UNSTABLE'
    >>> predicate(0).status.name
    'SUCCESS'
    >>> predicate(1).status.name
    'SUCCESS'

    >>> predicate = compile({'is_less_than': 0})
    >>> predicate(-1).status.name
    'SUCCESS'
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate(1).status.name
    'UNSTABLE'

    >>> predicate = compile({'is_less_than_or_equal_to': 0})
    >>> predicate(-1).status.name
    'SUCCESS'
    >>> predicate(0).status.name
    'SUCCESS'
    >>> predicate(1).status.name
    'UNSTABLE'

    >>> predicate = compile({'contains_string': '0'})
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate('012').status.name
    'SUCCESS'
    >>> predicate('123').status.name
    'UNSTABLE'

    >>> predicate = compile({'starts_with': 'AB'})
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate('ABC').status.name
    'SUCCESS'
    >>> predicate('ACB').status.name
    'UNSTABLE'

    >>> predicate = compile({'ends_with': 'BC'})
    >>> predicate(0).status.name
    'UNSTABLE'
    >>> predicate('ABC').status.name
    'SUCCESS'
    >>> predicate('ACB').status.name
    'UNSTABLE'

    >>> predicate = compile({'matches_regexp': '^A*B$'})
    >>> predicate('B').status.name
    'SUCCESS'
    >>> predicate('ACB').status.name
    'UNSTABLE'

    TODO: Should return `False` when the value type is not `str`.
    >>> predicate(0).status.name
    'FAILURE'
    """
    if isinstance(obj, str):
        matcher = _STATIC_MATCHER_MAP.get(obj)
        if not matcher:
            raise CompilationError(
                f'Invalid predicate: \'{obj}\''
            )
        return of_hamcrest_matcher(matcher)

    if len(obj) != 1:
        raise CompilationError(
            'Predicate must have only 1 element'
            f', but has {len(obj)}'
        )
    key, value = next(iter(obj.items()))
    if key in _VALUE_MATCHER_FUNCTION_MAP:
        matcher = _VALUE_MATCHER_FUNCTION_MAP[key](value)
        return of_hamcrest_matcher(matcher)

    raise CompilationError(f'Unrecognized predicate key: \'{key}\'')



"""Extraction compilation."""

from collections.abc import Mapping

from preacher.core.extraction import Extraction, with_jq
from .error import CompilationError


_EXTRACTION_MAP = {
    'jq': with_jq,
}
_EXTRACTION_KEYS = frozenset(_EXTRACTION_MAP.keys())


def compile(extraction_obj: Mapping) -> Extraction:
    """
    >>> compile({})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: ... has 0

    >>> compile({'jq': '.foo'})({'foo': 'bar'})
    'bar'
    """
    keys = _EXTRACTION_KEYS.intersection(extraction_obj.keys())
    if len(keys) != 1:
        raise CompilationError(
            'Description object must have only 1 valid extraction key'
            f', but has {len(keys)}'
        )
    key = next(iter(keys))

    return _EXTRACTION_MAP[key](extraction_obj[key])



"""Compilation."""

from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Iterator

from preacher.core.scenario import Scenario
from .error import CompilationError
from .scenario import compile_scenario


def compile(obj: Mapping) -> Iterator[Scenario]:
    """
    >>> scenarios = list(compile({}))
    >>> scenarios
    []

    >>> scenarios = compile({'scenarios': ''})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: scenarios ...

    >>> from unittest.mock import sentinel, patch
    >>> scenario_patch = patch(
    ...     f'{__name__}.compile_scenario',
    ...     return_value=sentinel.scenario,
    ... )
    >>> scenarios = compile({'scenarios': [{}, '']})
    >>> with scenario_patch as scenario_mock:
    ...     next(scenarios)
    ...     scenario_mock.call_args
    sentinel.scenario
    call({})
    >>> with scenario_patch as scenario_mock:
    ...     next(scenarios)
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Scenario ...
    """
    scenario_objs = obj.get('scenarios', [])
    if not isinstance(scenario_objs, list):
        raise CompilationError(f'scenarios must be a list: {scenario_objs}')
    return map(_compile_scenario, scenario_objs)


def _compile_scenario(obj: Any) -> Scenario:
    if not isinstance(obj, Mapping):
        raise CompilationError(f'Scenario must be a mapping: {obj}')
    return compile_scenario(obj)



"""Response description compilations."""

from collections.abc import Mapping
from typing import Any

from preacher.core.description import Description
from preacher.core.response_description import ResponseDescription
from .error import CompilationError
from .description import compile as compile_description
from .predicate import compile as compile_predicate


def compile(obj: Mapping) -> ResponseDescription:
    """
    >>> from unittest.mock import patch, sentinel
    >>> predicate_patch = patch(
    ...     f'{__name__}.compile_predicate',
    ...     return_value=sentinel.predicate,
    ... )
    >>> description_patch = patch(
    ...     f'{__name__}.compile_description',
    ...     return_value=sentinel.description,
    ... )

    >>> with predicate_patch as predicate_mock, \\
    ...      description_patch as description_mock:
    ...     response_description = compile({})
    ...     assert predicate_mock.call_count == 0
    ...     assert description_mock.call_count == 0
    >>> response_description.status_code_predicates
    []
    >>> response_description.body_descriptions
    []

    >>> compile({'body': 'str'})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: ResponseDescription.body ...

    >>> compile({'body': ['str']})
    Traceback (most recent call last):
        ...
    preacher.compilation.error.CompilationError: Description ...

    >>> with predicate_patch as predicate_mock, \\
    ...      description_patch as description_mock:
    ...     response_description = compile({
    ...         'status_code': 402,
    ...         'body': {'key1': 'value1'}}
    ...     )
    ...     predicate_mock.call_args_list
    ...     description_mock.call_args_list
    [call({'equals_to': 402})]
    [call({'key1': 'value1'})]
    >>> response_description.body_descriptions
    [sentinel.description]

    >>> with predicate_patch as predicate_mock, \\
    ...      description_patch as description_mock:
    ...     response_description = compile({
    ...         'status_code': [{'is_greater_than': 0}, {'is_less_than': 400}],
    ...         'body': [{'key1': 'value1'}, {'key2': 'value2'}],
    ...     })
    ...     predicate_mock.call_args_list
    ...     description_mock.call_args_list
    [call({'is_greater_than': 0}), call({'is_less_than': 400})]
    [call({'key1': 'value1'}), call({'key2': 'value2'})]
    >>> response_description.body_descriptions
    [sentinel.description, sentinel.description]
    """
    status_code_predicate_objs = obj.get('status_code', [])
    if isinstance(status_code_predicate_objs, int):
        status_code_predicate_objs = {'equals_to': status_code_predicate_objs}
    if not isinstance(status_code_predicate_objs, list):
        status_code_predicate_objs = [status_code_predicate_objs]
    status_code_predicates = [
        compile_predicate(obj) for obj in status_code_predicate_objs
    ]

    body_description_objs = obj.get('body', [])
    if isinstance(body_description_objs, Mapping):
        body_description_objs = [body_description_objs]
    if not isinstance(body_description_objs, list):
        raise CompilationError(
            'ResponseDescription.body must be a list or a mapping'
            f': {body_description_objs}'
        )
    body_descriptions = [
        _compile_description(body_description_obj)
        for body_description_obj in body_description_objs
    ]

    return ResponseDescription(
        status_code_predicates=status_code_predicates,
        body_descriptions=body_descriptions,
    )


def _compile_description(obj: Any) -> Description:
    if not isinstance(obj, Mapping):
        raise CompilationError(
            f'Description must be a mapping: {obj}'
        )
    return compile_description(obj)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['preacher',
 'preacher.app',
 'preacher.app.cli',
 'preacher.compilation',
 'preacher.core',
 'preacher.presentation']

package_data = \
{'': ['*']}

install_requires = \
['pyhamcrest>=1.9,<2.0',
 'pyjq>=2.3,<3.0',
 'requests>=2.21,<3.0',
 'ruamel.yaml>=0.15.89,<0.16.0']

entry_points = \
{'console_scripts': ['preacher-cli = preacher.app.cli.main:main']}

setup_kwargs = {
    'name': 'preacher',
    'version': '0.1.0',
    'description': 'A web API verification tool.',
    'long_description': '========\nPreacher\n========\n\n.. image:: https://travis-ci.org/ymoch/preacher.svg?branch=master\n    :target: https://travis-ci.org/ymoch/preacher\n.. image:: https://codecov.io/gh/ymoch/preacher/branch/master/graph/badge.svg\n    :target: https://codecov.io/gh/ymoch/preacher\n.. image:: https://img.shields.io/badge/python-3.7+-blue.svg\n    :target: https://www.python.org/\n\nPreacher verifies API servers,\nwhich requests to API servers and verify the response along to given scenarios.\n\nScenarios are written in `YAML`_ and `jq`_ queries\nso that any developers can write without learning toughly.\n\n\nUsage\n=====\n\nRequirements\n------------\nSupports only Python 3.7+.\n\n.. code-block:: sh\n\n    pip install preacher\n    preacher-cli scenario.yml\n\n    # Help command is available.\n    preacher-cli --help\n\n\nWriting Your Own Scenarios\n==========================\n\nExample\n-------\nHere is a simple configuration example.\n\n.. code-block:: yaml\n\n    scenarios:\n      - label: Simple\n        request:\n          path: /path/to/foo\n        response:\n          status_code: 200\n          body:\n            - describe:\n                jq: .foo\n              it:\n                equals_to: bar\n      - label: A Little Complecated\n        request:\n          path: /path/to/foo\n          params:\n            key1: value\n            key2:\n              - value1\n              - value2\n        response:\n          status_code:\n            - is_greater_than_or_equal_to: 200\n            - is_less_than: 400\n          body:\n            - describe:\n                jq: .foo\n              it:\n                - starts_with: x\n                - ends_wirh: y\n\nGrammer\n-------\n\nGlobal\n******\nA ``Configuration`` is written in `YAML`_.\nA ``Configuration`` is a mapping that consists of below:\n\n- scenarios: ``List<Scenario>``\n    - Scenarios.\n\n\nScenario\n********\nA ``Scenario`` is a mapping that consists of below:\n\n- label: ``String`` (Recommended)\n    - A label of this scenario.\n    - This field is actually optional but recommended to tell a scenario from another.\n- request: ``Request`` (Optional)\n    - A request.\n- response: ``ResponseDescription`` (Optional)\n    - A response description.\n\nRequest\n*******\nA ``Request`` is a mapping that consists of below:\n\n- path: ``String`` (Optional)\n    - A request path. The default value is ``\'\'``.\n- params: ``Mapping<String, String>`` (Optional)\n    - Query parameters as a mapping of keys to values.\n\nResponse Decription\n*******************\nA ``ResponseDescription`` is a mapping that consists of below:\n\n- status_code: ``Integer``, ``Predicate`` or ``List<Predicate>`` (Optional)\n    - Predicates that match a status code as an integer value.\n    - When given a number, that is equivalent to ``{"equals_to": it}``.\n- body: ``Description`` or ``List<Description>`` (Optional)\n    - Descriptions that descript the response body.\n\nDescription\n***********\nA ``Description`` is a mapping that consists of below:\n\n- describe: ``String`` or ``Extraction``\n    - An extraction process.\n    - When given a string, that is passed to the default extraction.\n- it: ``Predicate``, or ``List<Predicate>>``\n    - Predicates that match the extracted value.\n\nExtraction\n**********\nAn ``Extraction`` is a mapping that has one of below:\n\n- jq: ``String``\n    - A `jq`_ query.\n\nPredicate\n*********\nA ``Predicate`` is a string or a mapping. Allowed values are:\n\n- is_null\n- is_not_null\n- is_empty\n- is: ``Value``\n- equals_to: ``Value``\n- has_length: ``Integer``\n- is_greater_than: ``Numeric``\n- is_greater_than_or_equal_to: ``Numeric``\n- is_less_than: ``Numeric``\n- is_less_than_or_equal_to: ``Numeric``\n- contains_string: ``String``\n- starts_with: ``String``\n- ends_with: ``String``\n- matches_regexp: ``String``\n\n\n.. _YAML: https://yaml.org/\n.. _jq: https://stedolan.github.io/jq/\n',
    'author': 'Yu Mochizuki',
    'author_email': 'ymoch.dev@gmail.com',
    'url': 'https://github.com/ymoch/preacher',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Preacher CLI."""

from __future__ import annotations

import argparse
import logging
import sys

import ruamel.yaml as yaml

from preacher import __version__ as VERSION
from preacher.compilation import compile
from preacher.presentation.logger import LoggerPresentation
from .application import Application


HANDLER = logging.StreamHandler()
HANDLER.setLevel(logging.WARN)
LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(HANDLER)
LOGGER.setLevel(logging.INFO)


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'conf',
        nargs='+',
        help='config file paths'
    )
    parser.add_argument(
        '-v', '--version',
        action='version',
        version=VERSION,
    )
    parser.add_argument(
        '-u', '--url',
        metavar='url',
        help='specify the base URL',
        default='http://localhost:5000',
    )
    parser.add_argument(
        '-q', '--quiet',
        action='store_true',
        help='show details on the console only when an any issue occurs',
    )

    return parser.parse_args()


def main() -> None:
    """Main."""
    args = parse_args()

    logging_level = logging.INFO
    should_be_quiet = args.quiet
    if should_be_quiet:
        logging_level = logging.WARN
    HANDLER.setLevel(logging_level)

    base_url = args.url
    view = LoggerPresentation(LOGGER)
    app = Application(base_url=base_url, view=view)

    config_paths = args.conf
    for config_path in config_paths:
        with open(config_path) as config_file:
            config = yaml.safe_load(config_file)
        scenarios = compile(config)
        for scenario in scenarios:
            app.consume_scenario(scenario)

    if not app.is_succeeded:
        sys.exit(1)
